; LCD test for C8051F967

$NOMOD51
$include (C8051F960.inc)      
Period_low		EQU 0x00		; 0 for 1 sek
Period_high		EQU 0x80		; 0x80 for 1 sek
 
	DSEG 	AT 0x20			; data segment in direct addressable space
Status:	DS 1				; status bits for circular buffer
Counter:	DS 1				; counter for the # of measurements
dig3:		DS 1				; storage for digits
dig2:		DS 1
dig1:		DS 1
lcd3:		DS 1				; LCD data
lcd2:		DS 1
lcd1:		DS 1
lcd0:		DS 1

	CSEG 	AT 0				; interrupt vectors
	ljmp 	main  			; reset

	ORG	0x2B				; Timer2 ISR
	anl	TMR2CN, #0x3F		; clear Timer2 interrupt flags
	reti

	ORG	0xBB				; code section right after the int. vectors
	USING    0                    ; specify register bank 

main:       
	anl   PCA0MD, #NOT(0x40)      ; disable the WDT
	mov	SP, #0x7F			; setup stack in indirect storage area

	acall	Ports_Setup
	acall	Clock_Setup
	acall	Debug_Trap			; debug trap !!!
	acall	RTC_Setup
	acall	LCD_Setup

	mov	Counter, #0

loop: 					; MAIN LOOP
	mov	RTC0ADR, #RTC0XCN
	mov 	A, RTC0DAT

	mov 	PMU0CF, #CLEAR
	mov	PMU0CF, #(SLEEP OR RTCAWK) ; set wakeup sources
	nop   				; device is now awake
  	nop
	nop
	nop

   	mov	RTC0ADR, #RTC0CF		; clear SmaRTClock alarm IF
	mov	RTC0DAT, #(ALRM0EN OR AUTORST)

	mov	A, Counter
	inc	A				; increment display digit	
	anl 	A, #0x0F			; modulo 15
	mov	Counter, A			
	mov	dig3, A			; load digits
	mov	dig2, A
	mov	dig1, A
	acall	Display_Number

	sjmp	loop				; back to the main loop

;-------------------PROCEDURES-------------------------------------------
Ports_Setup:
    	mov  	P0MDOUT,   #0xFE		; configure all ports for output
	mov	P0,	     #0x01		; input on P1.0
	mov  	P0SKIP,    #0xFF

    	mov  	P1MDOUT,   #0xF3
	mov	P1MDIN,    #0xF3		; analog operation on XTAL pins
	mov	P1,        #0x0C
	mov  	P1SKIP,    #0xFF

    	mov  	P2MDOUT,   #0x8F		; LCD operation on P2.4-P2.6
	mov	P2MDIN,    #0x8F
	mov	P2,        #0x70    	
    	mov  	P2SKIP,    #0xFF

    	mov  	SFRPAGE,   #0x0F
    	mov  	P3MDOUT,   #0x80		; LCD operation on P3.0-P3.6
	mov	P3MDIN,    #0x80
	mov	P3,	     #0x7F
    	mov  	P4MDOUT,   #0x01
    	mov  	SFRPAGE,   #0x00
	mov  	XBR2,      #0x40		; enable XBar and weak pull-ups
	ret

;-----------------------------------------------------------------------
Clock_Setup:
	mov	RTC0ADR, #RTC0XCN		; disable Automatic Gain Control
	mov	RTC0DAT, #0x60		; and enable Bias Doubling
	mov	RTC0ADR, #RTC0XCF		; set 12 pF load capacitance
	mov	RTC0DAT, #0x84		; and enable atostep feature
	mov	RTC0ADR, #RTC0CN		; enable SmaRTClock
	mov	RTC0DAT, #(RTC0EN OR HSMODE OR RTC0TR)
	acall	Debug_Trap			; wait for XCO to start
	mov	RTC0ADR, #(0x40 OR RTC0XCN)
sc_loop1:
	mov 	A, RTC0DAT						
	jnb	ACC.4, sc_loop1		; verify that XOSC is stable
	mov	RTC0ADR, #(0x40 OR RTC0XCF)
sc_loop2:
	mov 	A, RTC0DAT						
	jnb	ACC.6, sc_loop2		; verify capacitance ready bit
	mov	RTC0ADR, #RTC0XCN		; enable Automatic Gain Control
	mov	RTC0DAT, #0xC0		; and disable Bias Doubling

;	mov	RTC0ADR, #RTC0XCN		; setup for LFO
;	mov	RTC0DAT, #LFOEN		; enable LFO
;	nop
;  	mov	RTC0ADR, #RTC0CN		; enable SmaRTClock
;	mov	RTC0DAT, #(RTC0EN OR HSMODE OR RTC0TR)  

	mov	PMU0MD,  #0			; enable POR supply monitor
	mov 	RSTSRC,  #0x06		; enable MCD and VDD monitor
	anl	OSCICN,  #NOT(0x80)	; disable internal precision oscillator
	mov	REG0CN,  #0			; disable precision oscillator bias
	mov	OSCXCN,  #0			; disable external oscillator
	mov	CLKSEL,  #0x03		; select SmaRTClock with 1:1 clock divider
	mov	A, CLKSEL			; wait for the divider setting to be applied
	jnb	ACC.7, $-2			
	ret

;-----------------------------------------------------------------------
Debug_Trap:
	mov	R2, #0xFF
	mov	A,  #0xFF
dt_loop:					; nested loops for approx. 10 sec delay
	nop
	djnz	ACC, dt_loop
	dec	R2
	mov	A, R2
	jnz	dt_loop	
	jnb	P0.0, $			; wait for slave debug trap is over
	ret

;-----------------------------------------------------------------------
RTC_Setup:	
	mov	CLKSEL, #0x04		; select LPO with 1:1 clock divider
	orl	FLSCL,  #BYPASS		; set the one-shot bypass bit
	mov	A, CLKSEL			; wait for the divider setting to be applied
	jnb	ACC.7, $-2

	mov 	RTC0ADR, #CAPTURE0	; clear RTC timer
	mov 	RTC0DAT, #0			; autoincrement address
	mov 	RTC0DAT, #0
	mov 	RTC0DAT, #0
	mov 	RTC0DAT, #0

	mov	RTC0ADR, #RTC0CN		; set timer value
	mov	RTC0DAT, #(RTC0EN OR HSMODE OR RTC0SET) 
	mov 	RTC0ADR, #(0x40 OR RTC0CN)
ts_loop:
	mov 	A, RTC0DAT						
	jb	ACC.1, ts_loop		; verify that timer is set

   	mov	RTC0ADR, #RTC0CN		; enable SmaRTClock timer
	mov	RTC0DAT, #(RTC0EN OR HSMODE OR RTC0TR)  

	mov	R6, #Period_low
	mov	R7, #Period_high
ALARM_Setup:
   	mov	RTC0ADR, #RTC0CF		; disable alarm while
   	mov	RTC0DAT, #0 		; updating the regs      
	mov 	RTC0ADR, #ALARM0B0	; load alarm regs       
	mov   RTC0DAT, R6
	mov	RTC0DAT, R7
	mov	RTC0DAT, #0 
	mov	RTC0DAT, #0
   	mov	RTC0ADR, #RTC0CF		; enable SmaRTClock alarm
	mov	RTC0DAT, #(ALRM0EN OR AUTORST)  
	ret

;-----------------------------------------------------------------------
LCD_Setup:
	mov	SFRPAGE,   #0x2		; switch to SFR page 0x2
;	mov	LCD0CN,    #0x05		; set 3-mux, 1/2 bias mode
	mov	LCD0CN,    #0x04		; set 3-mux, 1/3 bias mode
	mov	LCD0MSCN,  #0x56		; enable LCD clock and bias, clear LCD data
	mov	LCD0CNTRST,#0x0E		; set 2.72V contrast

	; Contrast Control Mode 1	; charge pump is permanently off
;	mov	LCD0MSCF, #0xFD		; bypass charge pump
;	mov	LCD0PWR,  #0x01		; set contrast mode 1
;	mov	LCD0VBMCN,#0x00		; disable VBAT monitor

	; Contrast Control Mode 2	; minimum contrast mode
;	mov	LCD0MSCF, #0xFD		; bypass charge pump
;	mov	LCD0PWR,  #0x09		; set contrast mode 2
;	mov	LCD0VBMCN,#0xC0		; enable VBAT monitor with offset=0

	; Contrast Control Mode 3	; constant contrast mode
	mov	LCD0MSCF, #0xFC		; do not bypass charge pump
	mov	LCD0PWR,  #0x09		; set contrast mode 3
	mov	LCD0VBMCN,#0xC0		; enable VBAT monitor with offset=0

	; Contrast Control Mode 4	; auto-bypass mode
;	mov	LCD0MSCF, #0xFC		; do not bypass charge pump
;	mov	LCD0PWR,  #0x01		; set contrast mode 4
;	mov	LCD0VBMCN,#0xC0		; enable VBAT monitor with offset=0

	mov	LCD0CF,   #0x20		; disable VLCD/VIO supply comparator
	mov	LCD0CLKDIVH, #0		; set refresh rate
	mov	LCD0CLKDIVL, #44	
	anl	LCD0MSCN, #NOT(0x02)	; reset clear LCD bit
	orl	LCD0MSCN, #0x01		; enable LCD driver
	mov	SFRPAGE,  #0x0		; switch to SFR page 0x0
	ret

;-----------------------------------------------------------------------
Display_Number:				; display dig3, dig2, dig1
	mov	DPTR, #table7seg		; load table pointer
	mov	A, dig3			; processing digit 3
	rl	A				; A = digit*2
	add	A, dig3			; A = digit*3
	mov	R4, A

	movc	A, @A+DPTR			; A = codes for seg. a
	anl	A, #0x01			; mask for digit 3			
	mov	lcd0, A
	inc	R4
	mov	A, R4
	movc	A, @A+DPTR			; A = codes for seg. def
	mov	lcd2, A
	inc	R4
	mov	A, R4
	movc	A, @A+DPTR			; A = codes for seg. cgb
	mov	lcd3, A

	mov	A, dig2			; processing digit 2
	rl	A				; A = digit*2
	add	A, dig2			; A = digit*3
	mov	R4, A

	movc	A, @A+DPTR			; A = codes for seg. a
	anl	A, #0x04			; mask for digit 2
	orl	lcd0, A
	inc	R4
	mov	A, R4
	movc	A, @A+DPTR			; A = codes for seg. def
	mov	lcd1, A
	inc	R4
	mov	A, R4
	movc	A, @A+DPTR			; A = codes for seg. cgb
	orl	lcd2, A

	mov	A, dig1			; processing digit 1
	rl	A				; A = digit*2
	add	A, dig1			; A = digit*3
	mov	R4, A

	movc	A, @A+DPTR			; A = codes for seg. a
	anl	A, #0x02			; mask for digit 1
	orl	lcd0, A
	inc	R4
	mov	A, R4
	movc	A, @A+DPTR			; A = codes for seg. def
	orl	lcd0, A
	inc	R4
	mov	A, R4
	movc	A, @A+DPTR			; A = codes for seg. cgb
	orl	lcd1, A

	mov	SFRPAGE, #0x2		; switch to SFR page 0x2
	mov	A, lcd0
	mov	LCD0D0, A
	mov	A, lcd1
	mov	LCD0D1, A
	mov	A, lcd2
	mov	LCD0D2, A
	mov	A, lcd3
	mov	LCD0D3, A
	mov	SFRPAGE, #0x0		; switch to SFR page 0x2
	ret

;========================================================================
table7seg: ; a  	def	cgb  		; 7-seg LCD table
	DB	0x07,	0x70, 0x05		; digit 0
	DB	0x00,	0x00, 0x05		; digit 1
	DB	0x07, 0x60, 0x03		; digit 2
	DB	0x07, 0x40, 0x07		; digit 3
	DB	0x00, 0x10, 0x07		; digit 4
	DB	0x07, 0x50, 0x06		; digit 5
	DB	0x07, 0x70, 0x06		; digit 6
	DB	0x07, 0x00, 0x05		; digit 7
	DB	0x07, 0x70, 0x07		; difit 8
	DB	0x07, 0x50, 0x07		; digit 9
	DB	0x07, 0x30, 0x07		; digit A
	DB	0x00, 0x70, 0x06		; digit b
	DB	0x07, 0x70, 0x00		; digit C
	DB	0x00, 0x60, 0x07		; digit d
	DB	0x07, 0x70, 0x02		; digit E
	DB	0x07, 0x30, 0x02		; digit F

END